from DisplayTarget import DisplayTarget
from utils import svg
from utils.datatypes import *

import gtk


#
# Target where sensors can draw on.
# Transparent colors are not supported due to missing support in GDK.
#
class TargetCanvas(DisplayTarget):

    def __init__(self, parent):

        DisplayTarget.__init__(self, parent)

        self.__widget = gtk.Image()
        self.__widget.show()

        self._register_property("graphics", TYPE_STRING,
                                self._setp_graphics, None)


    def get_widget(self): return self.__widget


    #
    # Transforms the given coordinates into buffer space.
    #
    def __transform_coords(self, x, y):

        width, height = self.get_geometry()[2:4]
        tx = (width / 2.0)  * (1.0 + float(x))
        ty = (height / 2.0) * (1.0 - float(y))

        return (tx, ty)


    def __make_style(self, foreground, fill):

        s = "stroke:" + foreground
        if (fill): s+= ";fill:" + foreground
        else: s+= ";fill:none"
        out = "style='%s'" % s
        return out


    #
    # Performs the given drawing operations. This is used for backwards
    # compatibility. New code should directly send SVG data.
    #
    def __draw_svg(self, commands):

        w, h = self.get_geometry()[2:4]
        out = "<svg width='%d' height='%d'>" % (w, h)
        current_fg = "rgb(0, 0, 0)"
        for c in commands:
            if (not c.strip()): continue
            parts = c.split()

            cmd, args = parts[0], parts[1:]

            if (cmd == "color"):
                color = args[0]
                gdkcolor = gtk.gdk.color_parse(color)
                current_fg = "rgb(%d, %d, %d)" \
                 % (gdkcolor.red >> 8, gdkcolor.green >> 8, gdkcolor.blue >> 8)

            elif (cmd == "line"):
                x1, y1, x2, y2 = args
                x1, y1 = self.__transform_coords(x1, y1)
                x2, y2 = self.__transform_coords(x2, y2)
                style = self.__make_style(current_fg, False)
                out += "<line x1='%f' y1='%f' x2='%f' y2='%f' %s/>" \
                       % (x1, y1, x2, y2, style)

            elif (cmd == "polygon"):
                fill = int(args[-1])
                style = self.__make_style(current_fg, fill)
                points = [ self.__transform_coords(args[i], args[i+1])
                           for i in range(0,len(args)-1, 2) ]
                if (points): path = "M%f %f " % (points.pop(0))
                while (points):
                    path += "L%f %f " % (points.pop(0))
                out += "<path d='%s' %s/>" % (path, style)

            elif (cmd == "rectangle"):
                x1, y1, x2, y2, fill = args
                style = self.__make_style(current_fg, fill)
                x1, y1 = self.__transform_coords(x1, y1)
                x2, y2 = self.__transform_coords(x2, y2)
                w = x2 - x1
                h = y2 - y1
                out += "<rect x='%f' y='%f' width='%f' height='%f' %s/>" \
                       % (x, y, w, h, style)

            #end if
        #end for
        out += "</svg>"
        svg.render(self.__widget, w, h, out)


    #
    # "graphics" property.
    #
    def _setp_graphics(self, key, value):

        # native SVG
        if (value.lstrip()[0] == "<"):
            w, h = self.get_geometry()[2:4]
            svg.render(self.__widget, w, h, value)

        # legacy graphics language
        else:
            value = value.split(",")
            self.__draw_svg(value)
