import libgtop
from Device import Device
from utils.Struct import Struct

import threading


class Total(Device):

    def __init__(self):

        Device.__init__(self)

        self.__cpu = Struct( total = 0.0,
                             user  = 0.0,
                             sys   = 0.0,
                             nice  = 0.0,
                             idle  = 0.0,
                             lock  = threading.Lock()
                             )


    def _get_cpu(self): return libgtop.cpu()


    def poll(self):

        """
        @return : cpu load
        @rtype  : float
        """
        self.__cpu.lock.acquire()

        try:
            cpu = self._get_cpu()
            totaldiff = cpu.total - self.__cpu.total
            load = float(cpu.frequency * ((cpu.user + cpu.sys + cpu.nice) -
                                          (self.__cpu.user + self.__cpu.sys + self.__cpu.nice))
                         / (totaldiff + 0.001) + 0.5)

            if (load > 100.0): load = 100.0

            self.__cpu.update(cpu)

            return load

        finally:
            self.__cpu.lock.release()




class CPU(Total):

    def __init__(self, n):

        Total.__init__(self)
        self.__n = n


    def _get_cpu(self): return libgtop.cpu().cpus[self.__n]
