from plugin.Interface import Interface

#
# Base class for controls. Every control has to be derived from this class.
#
class Control(object):

    # Warning : this is the only way to escape a Control
    # so be carefull when allowing a method to be called
    
    AUTHORIZED_METHODS = ('bind', )

    def __init__(self):

        # the handlers for property bindings
        self.__handlers = {}

        # flag indicating whether the control has been stopped
        self.__is_stopped = False


    def _update(self, property):
        """
        Issues a change of the given property.
        """

        lst = self.__handlers.get(property, [])
        for handler, args in lst:
            handler(getattr(self, property), *args)



    def bind(self, property, handler, *args):
        """
        Binds a handler to property changes.
        """

        self.__handlers.setdefault(property, []).append((handler, args))



    def _add_timer(self, interval, callback, *args):
        """
        Runs a timer callback at the given intervals.
        """

        def f():
            if self.__is_stopped: return False
            return bool(callback(*args))
            
        import gtk
        return gtk.timeout_add(interval, f)


    def _remove_timer(self, ident):
        import gtk
        gtk.timeout_remove(ident)


    #
    # Stops the control.
    #
    def stop(self):

        self.__is_stopped = True
        # remove bound handlers
        self.__handlers.clear()
        self._shutdown()


    #
    # Shutdown handler for controls. Controls should override this method,
    # if needed.
    #
    def _shutdown(self): pass


    def __interface(self):
        """
        Returns a description of the interface of this control.
        """

	return Interface.describe(self.__class__)

    interface = property(__interface, doc = "Interface description")


