from main import HOME

import gtk
import pango
import select
import textwrap
import thread
import time


# attributes for rendering text
_ATTR_DEBUG = {"foreground": "red",
               "weight": pango.WEIGHT_BOLD,
               "underline": pango.UNDERLINE_SINGLE}
_ATTR_DEPRECATED = {"foreground": "navy",
                    "style": pango.STYLE_ITALIC}
_ATTR_ERROR = {"foreground": "red",
               "weight": pango.WEIGHT_BOLD}
_ATTR_INFO = {"foreground": "green"}
_ATTR_NORMAL = {}



#
# Class for viewing log files.
#
class LogViewer(gtk.Dialog):

    def __init__(self, logfile):

        self.__running = False
        

        gtk.Dialog.__init__(self, _("Log Messages"), None, 0,
                            (gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE))
        self.set_has_separator(False)

        hbox = gtk.HBox()
        hbox.set_border_width(12)
        self.vbox.pack_start(hbox, True, True)

        self.__view = gtk.TextView()
        self.__view.set_size_request(-1, 240)
        self.__view.set_editable(False)
        self.__log = self.__view.get_buffer()

        self.__scrwin = gtk.ScrolledWindow()
        self.__scrwin.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self.__scrwin.add(self.__view)

        hbox.pack_end(self.__scrwin, True, True)
        self.show_all()

        def stop(*args): self.__halt()
        def destroy(*args): self.destroy()

        ok_button = self.get_children()[0].get_children()[-1]. \
                    get_children()[-1]
        self.connect("destroy", stop)
        ok_button.connect("clicked", destroy)

        self.__run(logfile)


    def __run(self, logfile):

        fd = open(logfile, "r")
        thread.start_new_thread(self.__watch_file, (fd,))
        self.__running = True


    def __halt(self):

        self.__running = False
        

    def __append_log(self, text, attributes):

        if (attributes):
            tag = self.__log.create_tag(None, **attributes)
            self.__log.insert_with_tags(self.__log.get_end_iter(), text + "\n",
                                        tag)
        else:
            self.__log.insert(self.__log.get_end_iter(), text + "\n")


    def __watch_file(self, fd):

        while (self.__running):
            import sys
            sys.stdout.flush()
            i, o, e = select.select([fd], [], [])
            for f in i:
                lines = f.readlines()
                for text in lines:
                    # analyze text
                    attr = self.__analyze_message(text)
                    for line in textwrap.wrap(text, 80):
                        gtk.idle_add(self.__append_log, line, attr)
            #end for
            time.sleep(0.5)
        #end while
        fd.close()


    #
    # Analyzes the given log message and returns the appropriate attribute.
    #
    def __analyze_message(self, message):
        
        if (message.find("DeprecationWarning: ") != -1):
            return _ATTR_DEPRECATED
        elif (message.find("Error: ") != -1):
            return _ATTR_ERROR
        elif (message.find("Info: ") != -1):
            return _ATTR_INFO
        elif (message.find("Debug: ") != -1):
            return _ATTR_DEBUG
        else:
            return _ATTR_NORMAL
