import os


# gDesklets home paths
HOME = os.path.abspath(os.path.join(os.path.dirname(__file__), os.pardir))
USERHOME = os.path.join(os.path.expanduser("~"), ".gdesklets")

# the path of the communication sockets
SOCKET_PATH = os.path.join(USERHOME, "sockets")

# the path of the log file
LOGFILE = os.path.join(USERHOME, "gdesklets" + os.environ["DISPLAY"] + ".log")

# the path of the positions file
POSITIONSFILE = os.path.join(USERHOME, "positions")

# the name of the purge key
PURGE_KEY = "_paths_to_purge_"

# for unset coordinates
UNSET_COORD = -1000

# paths where gDesklets looks for sensors and displays
SENSORPATHS = [os.path.join(USERHOME, "Sensors"),
               os.path.join(HOME, "Sensors")]
DISPLAYPATHS = [os.path.join(USERHOME, "Displays"),
                os.path.join(HOME, "Displays")]

NAME = "gDesklets"
VERSION = "0.31"
COPYRIGHT = u"Copyright \xa9 2003, 2004  The gDesklets Team"
DESCRIPTION = u"A desktop applet system for GNOME"
AUTHORS = (u"Martin Grimme <martin@pycage.de>",
           u"Christian Meyer <chrisime@gnome-de.org>",
           u"Jesse Andrews <jdandr2@cs.uky.edu>",
           u"S\xe9bastien Bacher <seb128@debian.org>",
           u"Beno\xeet Dejean <tazforever@dlfp.org>")
DOCUMENTERS = (u"Martin Grimme <martin@pycage.de>",)
ICON = os.path.join(HOME, "data", "gdesklets.png")


# setup i18n
from utils.i18n import Translator
_ = Translator("gdesklets", os.path.join(HOME, "locale"))


# create the gDesklets home dir
try:
    if (not os.path.exists(USERHOME)): os.makedirs(USERHOME)
except:
    pass


def init():

    import gnome.ui
    import gtk
    import signal

    # prepare GTK for threads
    gtk.threads_init()

    # init gnome
    chldhandler = signal.getsignal(signal.SIGCHLD)
    program = gnome.program_init(NAME, VERSION)

    # gnome.program_init has the bad new habit to modify the locale settings
    # (bug #135055); repair it
    import locale; locale.setlocale(locale.LC_NUMERIC, "C")

    # gnome.program_init < 1.99.18 breaks the SIGCHLD handler; gotta repair it
    signal.signal(signal.SIGCHLD, chldhandler)

    # install signal handler to quit on Ctrl-C
    signal.signal(signal.SIGINT, gtk.main_quit)

    #
    # Make sure that EventBoxes are transparent.
    #
    rc_style = """
    style "transparent" { bg_pixmap[NORMAL] = "<parent>"
                          bg_pixmap[INSENSITIVE] = "<parent>"
                          bg_pixmap[PRELIGHT] = "<parent>"
                          bg_pixmap[SELECTED] = "<parent>"
                          bg_pixmap[ACTIVE] = "<parent>"
                          }
    class "GtkEventBox" style "transparent"
    """

    gtk.rc_parse_string(rc_style)
